#!/usr/bin/perl -Tw
# Copyright 2002, Anders Nordby <anders@fix.no>
#
# Script to receive bounced Majordomo mails for approval.
# Part of "web interface for approving Majordomo messages".
#
# THIS SOFTWARE IS PROVIDED BY ANDERS NORDBY ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL ANDERS NORDBY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Casual users should only change this line:
my $ini = "/home/anders/bounceweb/bouncewebaddmsg.ini";

use Config::IniFiles;
use Fcntl;

my $cfg = new Config::IniFiles( -file => "$ini");
my $dbhost=$cfg->val( 'bounceweb', 'dbhost' );
my $dbuser=$cfg->val( 'bounceweb', 'dbuser' );
my $dbpw=$cfg->val( 'bounceweb', 'dbpw' );
my $dbname=$cfg->val( 'bounceweb', 'dbname' );
my $dbport=$cfg->val( 'bounceweb', 'dbport' );
my $mychars=$cfg->val( 'bounceweb', 'mychars' );
my $logfile=$cfg->val( 'bounceweb', 'logfile' );

use DBI;
use Getopt::Std;

sub validchars
{
	# 0: string 1: valid characters
	my $streng = $_[0];

	my $ok = 1;
	my $i = 0;
	while ($ok && $i < length($_[0])) {
		if (index($_[1], substr($_[0],$i,1)) == -1) {
			$ok = 0;
		}
		$i++;
	}
	return($ok);
}

sub cleanup {
	close LOGFILE;
	$dbh->disconnect;
}

$prog=$0;
getopts('l:');

$listname="";
my $mydate=localtime;

sysopen(LOGFILE, "$logfile", O_CREAT|O_APPEND|O_WRONLY);
print LOGFILE "--------------------------------------------------\n";
print LOGFILE "Receiving message \[$mydate\]\n";

$dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

if (!$dbh) {
	print LOGFILE "ERROR: $prog: could not connect to database.\n";
	cleanup;
	exit 1;
}

@data = <>;
if (!@data) {
	print LOGFILE "ERROR: no data.\n";
	cleanup;
	exit 1;
}

my $listname = $opt_l;
if (!$listname) {
	print LOGFILE "ERROR: No listname set.\n";
	cleanup;
	exit 1;
} elsif (!validchars($listname, $mychars)) {
	print LOGFILE "ERROR: Invalid characters in listname. Ignoring this mail.\n";
	cleanup;
	exit 1;
} else {
	print LOGFILE "OK listname: $listname\n";
}

my $dbq_listid = $dbh->prepare("select listid from lists where listname=\"$listname\";");
$dbq_listid->execute;
$listid=$dbq_listid->fetchrow_array;
$dbq_listid->finish;

if (!$listid) {
	print LOGFILE "ERROR: Could not find listid for list $listname. Ignoring mail.\n";
	cleanup;
        exit 1;
}

my $datastr = join("", @data);
if ($datastr =~ /\nSubject: BOUNCE .*: (.*Non-member submission from|Approval required:)/) {
	my $dbq = $dbh->prepare("insert into bouncemsg (msgno, listid, message, approved, approvecount, approvedbyuid, deleted) values (NULL, $listid, ?, NULL, NULL, NULL, 0);");
	$dbq->bind_param(1, $datastr);
	if (!$dbq->execute) {
		print LOGFILE "ERROR: SQL execution => $DBI::errstr\n";
	}
	$dbq->finish;
} else {
	print LOGFILE "ERROR: Could not find mail to be approved in input. Ignoring.\n";
}
cleanup;
