#!/usr/bin/perl -T
# Copyright 2002, Anders Nordby <anders@fix.no>
#
# Script to add a user.
# Part of "web interface for approving Majordomo messages".
#
# THIS SOFTWARE IS PROVIDED BY ANDERS NORDBY ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL ANDERS NORDBY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Casual users should only change this line:
my $ini = "/home/anders/bounceweb/bounceweb.ini";

use Config::IniFiles;
use Getopt::Std;

my $cfg = new Config::IniFiles( -file => "$ini");
my $dbhost=$cfg->val( 'bounceweb', 'dbhost' );
my $dbuser=$cfg->val( 'bounceweb', 'dbuser' );
my $dbpw=$cfg->val( 'bounceweb', 'dbpw' );
my $dbname=$cfg->val( 'bounceweb', 'dbname' );
my $dbport=$cfg->val( 'bounceweb', 'dbport' );
my $mychars=$cfg->val( 'bounceweb', 'mychars' );

use DBI;

sub validchars
{
	# 0: string 1: valid characters
	my $streng = $_[0];

	my $ok = 1;
	my $i = 0;
	while ($ok && $i < length($_[0])) {
		if (index($_[1], substr($_[0],$i,1)) == -1) {
			$ok = 0;
		}
		$i++;
	}
	return($ok);
}

sub salt {
	local($salt);               # initialization
	local($i, $rand);
	local(@itoa64) = ( '0' .. '9', 'a' .. 'z', 'A' .. 'Z' ); # 0 .. 63

	$rand = 0;
	# to64
	for ($i = 0; $i < 8; $i++) {
		srand(time + $rand + $$);
		$rand = rand(25*29*17 + $rand);
		$salt .=  $itoa64[$rand & $#itoa64];
	}
    return $salt;
}

sub usage {
	print "Usage:\t$prog -m <mode> [ -u <username> ] [ -p <password> ]\n";
	print "\t[ -l <listname> ]\n\n";
	print "The different modes and their required extra arguments are:\n\n";
	print "addlist: -l (add list)\n";
	print "adduser: -u -p (adduser with given password)\n";
	print "authuser: -u -l (authorize user for list)\n";
	print "deluser: -u (remove user)\n";
	print "dellist: -l (remove list)\n";
	print "delauth: -u -l (remove user authorization for list)\n";
	print "listauth: none (list which users are authorized with which lists)\n";
	print "listlists: none (list lists)\n";
	print "listusers: none (list users)\n";
	print "chpass: -u -p (set new password for user)\n";
}

sub perror {
	my $str = shift;
	print "ERROR: $str\n\n";
	usage;
	exit 1;
}

sub perrordb {
	print STDERR "ERROR: could not connect to database\n";
	exit 1;
}

sub yesno {
	my $question = shift;
	print "$question (y/n) ";

	chomp(my $answ = substr(<STDIN>, 0, 1));
	$answ =~ tr/Y/y/;
	if ($answ eq "y") {
		return 1;
	} else {
		return 0;
	}
}

sub adduser {
	my $newuser = $opt_u;
	my $newpass = $opt_p;

	if (!$newuser || !validchars($newuser, $mychars) || !$newpass || !validchars($newpass, $mychars)) {
		perror("Empty or illegal username entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $cryptpwd = crypt($newpass, &salt);
	my $dbq = $dbh->prepare("insert into users (userid, username, passwd) values (NULL, \"$newuser\", \"$cryptpwd\");");
	$dbq->execute;

	$dbq->finish;
	$dbh->disconnect;
}

sub addlist {
	my $newlist = $opt_l;

	if (!$newlist || !validchars($newlist, $mychars)) {
		perror("Empty or illegal listname entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq  = $dbh->prepare("insert into lists (listid, listname) values (NULL, \"$newlist\");");
	$dbq->execute;

	$dbq->finish;
	$dbh->disconnect;
}

sub authuser {
	my $authuser = $opt_u;
	my $authlist = $opt_l;

	if (!$authuser || !$authlist || !validchars($authuser, $mychars) || !validchars($authlist, $mychars)) {
		perror("Empty or illegal username/listname entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq_userid = $dbh->prepare("select userid from users where username=\"$authuser\";");
	$dbq_userid->execute;
	my $authuserid=$dbq_userid->fetchrow_array;  
	$dbq_userid->finish;

	my $dbq_listid = $dbh->prepare("select listid from lists where listname=\"$authlist\";");
	$dbq_listid->execute;
	my $authlistid=$dbq_listid->fetchrow_array;
	$dbq_listid->finish;

	if (!$authuserid || !$authlistid) {
		$dbh->disconnect;
		perror("Could not find username/listname entered.");
	} else {
		chomp($authuserid);
		chomp($authlistid);

		my $dbq = $dbh->prepare("insert into acl (userid, listid) values ($authuserid, $authlistid);");
		$dbq->execute;
		$dbq->finish;
	}
	$dbh->disconnect;
}

sub deluser {
	my $deluser = $opt_u;

	if (!$deluser || !validchars($deluser, $mychars)) {
		perror("Empty or illegal username entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq_userid = $dbh->prepare("select userid from users where username=\"$deluser\";");
	$dbq_userid->execute;
	my $deluserid=$dbq_userid->fetchrow_array;  
	$dbq_userid->finish;

	if (!$deluserid) {
		$dbh->disconnect;
		perror("Could not find username entered.");
	} else {
		if (yesno("This will also delete all list authorizations for this user. Are you sure you\nwant to continue?")) {
			chomp($deluserid);

			print "OK, deleting user $deluser.\n";

			my $dbq = $dbh->prepare("delete from acl where userid=\"$deluserid\";");
			$dbq->execute;
			$dbq->finish;

			my $dbq = $dbh->prepare("delete from users where userid=\"$deluserid\";");
			$dbq->execute;
			$dbq->finish;

			print "Done.\n";
		}
	}
	$dbh->disconnect;
}

sub dellist {
	my $dlistname = $opt_l;

	if (!$dlistname || !validchars($dlistname, $mychars)) {
		perror("Empty or illegal listname entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq_listid = $dbh->prepare("select listid from lists where listname=\"$dlistname\";");
	$dbq_listid->execute;
	my $dlistid=$dbq_listid->fetchrow_array;  
	$dbq_listid->finish;

	if (!$dlistid) {
		$dbh->disconnect;
		perror("Could not find listname entered.");
	} else {
		if (yesno("This will also delete all authorizations for this list. Are you sure you\nwant to continue?")) {
			chomp($dlistid);

			print "OK, deleting list $dlistname.\n";

			my $dbq = $dbh->prepare("delete from acl where listid=\"$dlistid\";");
			$dbq->execute;
			$dbq->finish;

			my $dbq = $dbh->prepare("delete from lists where listid=\"$dlistid\";");
			$dbq->execute;
			$dbq->finish;
		}
	}
	$dbh->disconnect;
}

sub delauth {
	my $deluser = $opt_u;
	my $dlistname = $opt_l;

	if (!$dlistname || !validchars($dlistname, $mychars) || !$deluser || !validchars($deluser, $mychars)) {
		perror("Empty or illegal listname/username entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq_listid = $dbh->prepare("select listid from lists where listname=\"$dlistname\";");
	$dbq_listid->execute;
	my $dlistid=$dbq_listid->fetchrow_array;  
	$dbq_listid->finish;

	my $dbq_userid = $dbh->prepare("select userid from users where username=\"$deluser\";");
	$dbq_userid->execute;
	my $deluserid=$dbq_userid->fetchrow_array;  
	$dbq_userid->finish;

	if (!$dlistid || !$deluserid) {
		$dbh->disconnect;
		perror("Could not find listname/username entered.");
	} else {
		chomp($dlistid);
		chomp($deluserid);

		print "OK, deleting authorization for user $deluser on list $dlistname.\n";

		my $dbq = $dbh->prepare("delete from acl where listid=\"$dlistid\" and userid=\"$deluserid\";");
		$dbq->execute;
		$dbq->finish;
	}
	$dbh->disconnect;
}

sub chpass {
	my $passuser = $opt_u;
	my $newpass = $opt_p;

	if (!$passuser || !validchars($passuser, $mychars) || !$newpass || !validchars($newpass, $mychars)) {
		perror("Empty or illegal username/password entered.");
	}

	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq_userid = $dbh->prepare("select userid from users where username=\"$passuser\";");
	$dbq_userid->execute;
	my $passuserid=$dbq_userid->fetchrow_array;  
	$dbq_userid->finish;

	if (!$passuserid) {
		$dbh->disconnect;
		perror("Could not find username entered.");
	} else {
		chomp($dlistid);

		my $cryptpwd = crypt($newpass, &salt);
		my $dbq = $dbh->prepare("update users set passwd=\"$cryptpwd\" where userid=\"$passuserid\";");
		$dbq->execute;
		$dbq->finish;

		print "OK, password for user $passuser is changed.\n";
	}
	$dbh->disconnect;
}

sub listusers {
	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq = $dbh->prepare("select username as username, passwd as password from users;");
	$dbq->execute;

	print "username;cryptpassword\n\n";
	while (my $row = $dbq->fetchrow_hashref()) {
		print "$row->{username};$row->{password}\n";
	}

	$dbq->finish;
	$dbh->disconnect;
}

sub listlists {
	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq = $dbh->prepare("select listname as listname from lists;");
	$dbq->execute;

	print "listname\n\n";
	while (my $row = $dbq->fetchrow_hashref()) {
		print "$row->{listname}\n";
	}

	$dbq->finish;
	$dbh->disconnect;
}

sub listauth {
	my $dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

	if (!$dbh) { perrordb; }

	my $dbq = $dbh->prepare("select lists.listname as listname, users.username from lists, acl, users where lists.listid = acl.listid and users.userid = acl.userid;");
	$dbq->execute;

	print "username;listname\n\n";
	while (my $row = $dbq->fetchrow_hashref()) {
		print "$row->{username};$row->{listname}\n";
	}

	$dbq->finish;
	$dbh->disconnect;
}

$prog=$0;
getopts('m:u:p:l:');

for ($opt_m) {
	/^adduser$/	and do { adduser; next; };
	/^addlist$/	and do { addlist; next; };
	/^authuser$/	and do { authuser; next; };
	/^deluser$/	and do { deluser; next; };
	/^dellist$/	and do { dellist; next; };
	/^delauth$/	and do { delauth; next; };
	/^chpass$/	and do { chpass; next; };
	/^listusers$/	and do { listusers; next; };
	/^listlists$/	and do { listlists; next; };
	/^listauth$/	and do { listauth; next; };
	perror("Unknown mode.");
}
